@@ -378,8 +378,10 @@ error:
 	case CMD_STATUS_FAILED:
 		goto failure;
 
-	default:
+	case CMD_STATUS_TRIGGERED:
+	case CMD_STATUS_DONE:
 		l_error("Unknown command status.");
+		break;
 	}
 
 	if (interactive_mode)
@@ -509,8 +511,14 @@ void command_process_prompt(char **argv, int argc)
 		return;
 
 	if (!interactive_mode) {
+		if (command_match_misc_commands(argv, argc)) {
+			exit_status = EXIT_SUCCESS;
+			goto quit;
+		}
+
 		display_error("Invalid command\n");
 		exit_status = EXIT_FAILURE;
+quit:
 		l_main_quit();
 		return;
 	}
