@@ -185,8 +185,23 @@ void display_refresh_set_cmd(const char *family, const char *entity,
 		L_AUTO_FREE_VAR(char *, args);
 		char *prompt;
 
-		for (i = 0; i < argc; i++)
-			l_string_append_printf(buf, "'%s' ", argv[i]);
+		for (i = 0; i < argc; i++) {
+			bool needs_quotes = false;
+			char *p = argv[i];
+
+			for (p = argv[i]; *p != '\0'; p++) {
+				if (*p != ' ')
+					continue;
+
+				needs_quotes = true;
+				break;
+			}
+
+			if (needs_quotes)
+				l_string_append_printf(buf, "\"%s\" ", argv[i]);
+			else
+				l_string_append_printf(buf, "%s ", argv[i]);
+		}
 
 		args = l_string_unwrap(buf);
 
@@ -568,10 +583,6 @@ void display_agent_prompt_release(const char *label)
 	agent_saved_input = NULL;
 
 	rl_set_prompt(IWD_PROMPT);
-
-	rl_forced_update_display();
-
-	display("\r");
 }
 
 void display_quit(void)
