@@ -44,23 +44,28 @@
 #include "src/handshake.h"
 #include "src/ap.h"
 #include "src/dbus.h"
+#include "src/nl80211util.h"
 
 struct ap_state {
 	struct netdev *netdev;
 	char *ssid;
 	int channel;
 	unsigned int ciphers;
+	enum ie_rsn_cipher_suite group_cipher;
 	uint32_t beacon_interval;
 	struct l_uintset *rates;
 	uint8_t pmk[32];
 	struct l_queue *frame_watch_ids;
 	uint32_t start_stop_cmd_id;
+	uint8_t gtk[CRYPTO_MAX_GTK_LEN];
+	uint8_t gtk_index;
 
 	uint16_t last_aid;
 	struct l_queue *sta_states;
 
 	struct l_dbus_message *pending;
 	bool started : 1;
+	bool gtk_set : 1;
 };
 
 struct sta_state {
@@ -76,6 +81,7 @@ struct sta_state {
 	uint8_t *assoc_rsne;
 	struct eapol_sm *sm;
 	struct handshake_state *hs;
+	uint32_t gtk_query_cmd_id;
 };
 
 static struct l_genl_family *nl80211 = NULL;
@@ -91,6 +97,9 @@ static void ap_sta_free(void *data)
 	if (sta->assoc_resp_cmd_id)
 		l_genl_family_cancel(nl80211, sta->assoc_resp_cmd_id);
 
+	if (sta->gtk_query_cmd_id)
+		l_genl_family_cancel(nl80211, sta->gtk_query_cmd_id);
+
 	if (sta->sm)
 		eapol_sm_free(sta->sm);
 
@@ -153,6 +162,11 @@ static void ap_del_station(struct sta_state *sta, uint16_t reason,
 	sta->associated = false;
 	sta->rsna = false;
 
+	if (sta->gtk_query_cmd_id) {
+		l_genl_family_cancel(nl80211, sta->gtk_query_cmd_id);
+		sta->gtk_query_cmd_id = 0;
+	}
+
 	if (sta->sm)
 		eapol_sm_free(sta->sm);
 
@@ -207,20 +221,13 @@ static void ap_drop_rsna(struct sta_state *sta)
 {
 	struct l_genl_msg *msg;
 	uint32_t ifindex = netdev_get_ifindex(sta->ap->netdev);
-	struct nl80211_sta_flag_update flags = {
-		.mask = (1 << NL80211_STA_FLAG_AUTHORIZED) |
-			(1 << NL80211_STA_FLAG_MFP),
-		.set = 0,
-	};
 	uint8_t key_id = 0;
 
 	sta->rsna = false;
 
-	msg = l_genl_msg_new_sized(NL80211_CMD_SET_STATION, 128);
-	l_genl_msg_append_attr(msg, NL80211_ATTR_IFINDEX, 4, &ifindex);
-	l_genl_msg_append_attr(msg, NL80211_ATTR_MAC, 6, sta->addr);
+	msg = nl80211_build_set_station_unauthorized(ifindex, sta->addr);
+
 	l_genl_msg_append_attr(msg, NL80211_ATTR_STA_AID, 2, &sta->aid);
-	l_genl_msg_append_attr(msg, NL80211_ATTR_STA_FLAGS2, 8, &flags);
 
 	if (!l_genl_family_send(nl80211, msg, ap_set_sta_cb, NULL, NULL)) {
 		l_genl_msg_unref(msg);
@@ -252,7 +259,7 @@ static void ap_set_rsn_info(struct ap_state *ap, struct ie_rsn_info *rsn)
 	memset(rsn, 0, sizeof(*rsn));
 	rsn->akm_suites = IE_RSN_AKM_SUITE_PSK;
 	rsn->pairwise_ciphers = ap->ciphers;
-	rsn->group_cipher = IE_RSN_CIPHER_SUITE_NO_GROUP_TRAFFIC;
+	rsn->group_cipher = ap->group_cipher;
 }
 
 /*
@@ -390,9 +397,8 @@ static void ap_handshake_event(struct handshake_state *hs,
 	}
 }
 
-static void ap_associate_sta_cb(struct l_genl_msg *msg, void *user_data)
+static void ap_start_rsna(struct sta_state *sta, const uint8_t *gtk_rsc)
 {
-	struct sta_state *sta = user_data;
 	struct ap_state *ap = sta->ap;
 	struct netdev *netdev = sta->ap->netdev;
 	const uint8_t *own_addr = netdev_get_address(netdev);
@@ -400,12 +406,6 @@ static void ap_associate_sta_cb(struct l_genl_msg *msg, void *user_data)
 	struct ie_rsn_info rsn;
 	uint8_t bss_rsne[24];
 
-	if (l_genl_msg_get_error(msg) < 0) {
-		l_error("NEW_STATION/SET_STATION failed: %i",
-			l_genl_msg_get_error(msg));
-		return;
-	}
-
 	memset(&bss, 0, sizeof(bss));
 
 	ap_set_rsn_info(ap, &rsn);
@@ -427,6 +427,10 @@ static void ap_associate_sta_cb(struct l_genl_msg *msg, void *user_data)
 	handshake_state_set_authenticator_address(sta->hs, own_addr);
 	handshake_state_set_supplicant_address(sta->hs, sta->addr);
 
+	if (gtk_rsc)
+		handshake_state_set_gtk(sta->hs, ap->gtk, ap->gtk_index,
+					gtk_rsc);
+
 	sta->sm = eapol_sm_new(sta->hs);
 	if (!sta->sm) {
 		handshake_state_free(sta->hs);
@@ -446,10 +450,43 @@ error:
 	ap_del_station(sta, MMPDU_REASON_CODE_UNSPECIFIED, true);
 }
 
-static void ap_associate_sta(struct ap_state *ap, struct sta_state *sta)
+static void ap_gtk_query_cb(struct l_genl_msg *msg, void *user_data)
+{
+	struct sta_state *sta = user_data;
+	const void *gtk_rsc;
+
+	sta->gtk_query_cmd_id = 0;
+
+	gtk_rsc = nl80211_parse_get_key_seq(msg);
+	if (!gtk_rsc)
+		goto error;
+
+	ap_start_rsna(sta, gtk_rsc);
+	return;
+
+error:
+	ap_del_station(sta, MMPDU_REASON_CODE_UNSPECIFIED, true);
+}
+
+static struct l_genl_msg *ap_build_cmd_del_key(struct ap_state *ap)
+{
+	uint32_t ifindex = netdev_get_ifindex(ap->netdev);
+	struct l_genl_msg *msg;
+
+	msg = l_genl_msg_new_sized(NL80211_CMD_DEL_KEY, 128);
+
+	l_genl_msg_append_attr(msg, NL80211_ATTR_IFINDEX, 4, &ifindex);
+	l_genl_msg_enter_nested(msg, NL80211_ATTR_KEY);
+	l_genl_msg_append_attr(msg, NL80211_KEY_IDX, 1, &ap->gtk_index);
+	l_genl_msg_leave_nested(msg);
+
+	return msg;
+}
+
+static struct l_genl_msg *ap_build_cmd_new_station(struct sta_state *sta)
 {
 	struct l_genl_msg *msg;
-	uint32_t ifindex = netdev_get_ifindex(ap->netdev);
+	uint32_t ifindex = netdev_get_ifindex(sta->ap->netdev);
 	/*
 	 * This should hopefully work both with and without
 	 * NL80211_FEATURE_FULL_AP_CLIENT_STATE.
@@ -462,13 +499,134 @@ static void ap_associate_sta(struct ap_state *ap, struct sta_state *sta)
 		.set = (1 << NL80211_STA_FLAG_AUTHENTICATED) |
 			(1 << NL80211_STA_FLAG_ASSOCIATED),
 	};
+
+	msg = l_genl_msg_new_sized(NL80211_CMD_NEW_STATION, 300);
+
+	l_genl_msg_append_attr(msg, NL80211_ATTR_IFINDEX, 4, &ifindex);
+	l_genl_msg_append_attr(msg, NL80211_ATTR_MAC, 6, sta->addr);
+	l_genl_msg_append_attr(msg, NL80211_ATTR_STA_FLAGS2, 8, &flags);
+
+	return msg;
+}
+
+static void ap_gtk_op_cb(struct l_genl_msg *msg, void *user_data)
+{
+	if (l_genl_msg_get_error(msg) < 0) {
+		uint8_t cmd = l_genl_msg_get_command(msg);
+		const char *cmd_name =
+			cmd == NL80211_CMD_NEW_KEY ? "NEW_KEY" :
+			cmd == NL80211_CMD_SET_KEY ? "SET_KEY" :
+			"DEL_KEY";
+
+		l_error("%s failed for the GTK: %i",
+			cmd_name, l_genl_msg_get_error(msg));
+	}
+}
+
+static void ap_associate_sta_cb(struct l_genl_msg *msg, void *user_data)
+{
+	struct sta_state *sta = user_data;
+	struct ap_state *ap = sta->ap;
+
+	if (l_genl_msg_get_error(msg) < 0) {
+		l_error("NEW_STATION/SET_STATION failed: %i",
+			l_genl_msg_get_error(msg));
+		return;
+	}
+
+	/*
+	 * Set up the group key.  If this is our first STA then we have
+	 * to add the new GTK to the kernel.  In theory we should be
+	 * able to supply our own RSC (e.g. generated randomly) and use it
+	 * immediately for our 4-Way Handshake without querying the kernel.
+	 * However NL80211_CMD_NEW_KEY only lets us set the receive RSC --
+	 * the Rx PN for CCMP and the Rx IV for TKIP -- and the
+	 * transmit RSC always starts as all zeros.  There's effectively
+	 * no way to set the Tx RSC or query the Rx RSC through nl80211.
+	 * So we query the Tx RSC in both scenarios just in case some
+	 * driver/hardware uses a different initial Tx RSC.
+	 *
+	 * Optimally we would get called back by the EAPoL state machine
+	 * only when building the step 3 of 4 message to query the RSC as
+	 * late as possible but that would complicate EAPoL.
+	 */
+	if (ap->group_cipher != IE_RSN_CIPHER_SUITE_NO_GROUP_TRAFFIC &&
+			!ap->gtk_set) {
+		enum crypto_cipher group_cipher =
+			ie_rsn_cipher_suite_to_cipher(ap->group_cipher);
+		int gtk_len = crypto_cipher_key_len(group_cipher);
+
+		/*
+		 * Generate our GTK.  Not following the example derivation
+		 * method in 802.11-2016 section 12.7.1.4 because a simple
+		 * l_getrandom is just as good.
+		 */
+		l_getrandom(ap->gtk, gtk_len);
+		ap->gtk_index = 1;
+
+		msg = nl80211_build_new_key_group(
+						netdev_get_ifindex(ap->netdev),
+						group_cipher, ap->gtk_index,
+						ap->gtk, gtk_len, NULL,
+						0, NULL);
+
+		if (!l_genl_family_send(nl80211, msg, ap_gtk_op_cb, NULL,
+					NULL)) {
+			l_genl_msg_unref(msg);
+			l_error("Issuing NEW_KEY failed");
+			goto error;
+		}
+
+		msg = nl80211_build_set_key(netdev_get_ifindex(ap->netdev),
+						ap->gtk_index);
+		if (!l_genl_family_send(nl80211, msg, ap_gtk_op_cb, NULL,
+					NULL)) {
+			l_genl_msg_unref(msg);
+			l_error("Issuing SET_KEY failed");
+			goto error;
+		}
+
+		/*
+		 * Set the flag now because any new associating STA will
+		 * just use NL80211_CMD_GET_KEY from now.
+		 */
+		ap->gtk_set = true;
+	}
+
+	if (ap->group_cipher == IE_RSN_CIPHER_SUITE_NO_GROUP_TRAFFIC)
+		ap_start_rsna(sta, NULL);
+	else {
+		msg = nl80211_build_get_key(netdev_get_ifindex(ap->netdev),
+					ap->gtk_index);
+		sta->gtk_query_cmd_id = l_genl_family_send(nl80211, msg,
+								ap_gtk_query_cb,
+								sta, NULL);
+		if (!sta->gtk_query_cmd_id) {
+			l_genl_msg_unref(msg);
+			l_error("Issuing GET_KEY failed");
+			goto error;
+		}
+	}
+
+	return;
+
+error:
+	ap_del_station(sta, MMPDU_REASON_CODE_UNSPECIFIED, true);
+}
+
+static void ap_associate_sta(struct ap_state *ap, struct sta_state *sta)
+{
+	struct l_genl_msg *msg;
+	uint32_t ifindex = netdev_get_ifindex(ap->netdev);
+
 	uint8_t rates[256];
 	uint32_t r, minr, maxr, count = 0;
 	uint16_t capability = l_get_le16(&sta->capability);
-	uint8_t cmd = NL80211_CMD_NEW_STATION;
 
 	if (sta->associated)
-		cmd = NL80211_CMD_SET_STATION;
+		msg = nl80211_build_set_station_associated(ifindex, sta->addr);
+	else
+		msg = ap_build_cmd_new_station(sta);
 
 	sta->associated = true;
 	sta->rsna = false;
@@ -480,11 +638,7 @@ static void ap_associate_sta(struct ap_state *ap, struct sta_state *sta)
 		if (l_uintset_contains(sta->rates, r))
 			rates[count++] = r;
 
-	msg = l_genl_msg_new_sized(cmd, 300);
-	l_genl_msg_append_attr(msg, NL80211_ATTR_IFINDEX, 4, &ifindex);
-	l_genl_msg_append_attr(msg, NL80211_ATTR_MAC, 6, sta->addr);
 	l_genl_msg_append_attr(msg, NL80211_ATTR_STA_AID, 2, &sta->aid);
-	l_genl_msg_append_attr(msg, NL80211_ATTR_STA_FLAGS2, 8, &flags);
 	l_genl_msg_append_attr(msg, NL80211_ATTR_STA_SUPPORTED_RATES,
 				count, &rates);
 	l_genl_msg_append_attr(msg, NL80211_ATTR_STA_LISTEN_INTERVAL, 2,
@@ -494,7 +648,7 @@ static void ap_associate_sta(struct ap_state *ap, struct sta_state *sta)
 
 	if (!l_genl_family_send(nl80211, msg, ap_associate_sta_cb, sta, NULL)) {
 		l_genl_msg_unref(msg);
-		if (cmd == NL80211_CMD_NEW_STATION)
+		if (l_genl_msg_get_command(msg) == NL80211_CMD_NEW_STATION)
 			l_error("Issuing NEW_STATION failed");
 		else
 			l_error("Issuing SET_STATION failed");
@@ -1235,6 +1389,7 @@ static int ap_start(struct ap_state *ap, const char *ssid, const char *psk,
 	ap->channel = 6;
 	/* TODO: Add all ciphers supported by wiphy */
 	ap->ciphers = wiphy_select_cipher(wiphy, 0xffff);
+	ap->group_cipher = wiphy_select_cipher(wiphy, 0xffff);
 	ap->beacon_interval = 100;
 	/* TODO: Use actual supported rates */
 	ap->rates = l_uintset_new(200);
@@ -1356,6 +1511,19 @@ static int ap_stop(struct ap_state *ap, struct l_dbus_message *message)
 		return -EIO;
 	}
 
+	if (ap->gtk_set) {
+		struct l_genl_msg *msg;
+
+		ap->gtk_set = false;
+
+		msg = ap_build_cmd_del_key(ap);
+		if (!l_genl_family_send(nl80211, msg, ap_gtk_op_cb, NULL,
+					NULL)) {
+			l_genl_msg_unref(msg);
+			l_error("Issuing DEL_KEY failed");
+		}
+	}
+
 	ap->pending = l_dbus_message_ref(message);
 
 	return 0;
@@ -1452,15 +1620,11 @@ static void ap_remove_interface(struct netdev *netdev)
 static void ap_netdev_watch(struct netdev *netdev,
 				enum netdev_watch_event event, void *userdata)
 {
-	struct device *device = netdev_get_device(netdev);
-
-	if (!device)
-		return;
-
 	switch (event) {
 	case NETDEV_WATCH_EVENT_UP:
 	case NETDEV_WATCH_EVENT_NEW:
-		if (netdev_get_iftype(netdev) == NETDEV_IFTYPE_AP)
+		if (netdev_get_iftype(netdev) == NETDEV_IFTYPE_AP &&
+				netdev_get_is_up(netdev))
 			ap_add_interface(netdev);
 		break;
 	case NETDEV_WATCH_EVENT_DOWN:
