@@ -33,23 +33,20 @@
 
 #include <ell/ell.h>
 
-#include "iwd.h"
-#include "storage.h"
-#include "common.h"
-#include "network.h"
-#include "dbus.h"
-#include "knownnetworks.h"
+#include "src/iwd.h"
+#include "src/storage.h"
+#include "src/common.h"
+#include "src/network.h"
+#include "src/dbus.h"
+#include "src/knownnetworks.h"
 
 static struct l_queue *known_networks;
 static size_t num_known_hidden_networks;
-static struct l_fswatch *storage_dir_watch;
+static struct l_dir_watch *storage_dir_watch;
 
-static int timespec_compare(const void *a, const void *b, void *user_data)
+static int timespec_compare(const struct timespec *tsa,
+				const struct timespec *tsb)
 {
-	const struct network_info *ni_a = a;
-	const struct network_info *ni_b = b;
-	const struct timespec *tsa = &ni_a->connected_time;
-	const struct timespec *tsb = &ni_b->connected_time;
 
 	if (tsa->tv_sec > tsb->tv_sec)
 		return -1;
@@ -66,8 +63,15 @@ static int timespec_compare(const void *a, const void *b, void *user_data)
 	return 0;
 }
 
-static const char *iwd_known_network_get_path(
-					const struct network_info *network)
+static int connected_time_compare(const void *a, const void *b, void *user_data)
+{
+	const struct network_info *ni_a = a;
+	const struct network_info *ni_b = b;
+
+	return timespec_compare(&ni_a->connected_time, &ni_b->connected_time);
+}
+
+const char *known_network_get_path(const struct network_info *network)
 {
 	static char path[256];
 	unsigned int pos = 0, i;
@@ -86,7 +90,7 @@ static const char *iwd_known_network_get_path(
 
 static void known_network_register_dbus(struct network_info *network)
 {
-	const char *path = iwd_known_network_get_path(network);
+	const char *path = known_network_get_path(network);
 
 	if (!l_dbus_object_add_interface(dbus_get_bus(), path,
 					IWD_KNOWN_NETWORK_INTERFACE, network))
@@ -113,15 +117,16 @@ static void known_network_update(struct network_info *orig_network,
 	else
 		network = network_info_add_known(ssid, security);
 
-	if (timespec_compare(&network->connected_time, connected_time, NULL) &&
+	if (timespec_compare(&network->connected_time, connected_time) &&
 			orig_network) {
 		l_dbus_property_changed(dbus_get_bus(),
-					iwd_known_network_get_path(network),
+					known_network_get_path(network),
 					IWD_KNOWN_NETWORK_INTERFACE,
 					"LastConnectedTime");
 
 		l_queue_remove(known_networks, network);
-		l_queue_insert(known_networks, network, timespec_compare, NULL);
+		l_queue_insert(known_networks, network, connected_time_compare,
+				NULL);
 	}
 
 	memcpy(&network->connected_time, connected_time,
@@ -134,7 +139,7 @@ static void known_network_update(struct network_info *orig_network,
 
 	if (network->is_hidden != is_hidden && orig_network)
 		l_dbus_property_changed(dbus_get_bus(),
-					iwd_known_network_get_path(network),
+					known_network_get_path(network),
 					IWD_KNOWN_NETWORK_INTERFACE,
 					"Hidden");
 
@@ -146,7 +151,7 @@ static void known_network_update(struct network_info *orig_network,
 	if (orig_network)
 		return;
 
-	l_queue_insert(known_networks, network, timespec_compare, NULL);
+	l_queue_insert(known_networks, network, connected_time_compare, NULL);
 
 	known_network_register_dbus(network);
 }
@@ -280,7 +285,7 @@ static void known_network_removed(struct network_info *network)
 
 	l_queue_remove(known_networks, network);
 	l_dbus_unregister_object(dbus_get_bus(),
-					iwd_known_network_get_path(network));
+					known_network_get_path(network));
 
 	/*
 	 * network_info_forget_known will either re-add the network_info to
@@ -289,10 +294,9 @@ static void known_network_removed(struct network_info *network)
 	network_info_forget_known(network);
 }
 
-static void known_networks_watch_cb(struct l_fswatch *watch,
-					const char *filename,
-					enum l_fswatch_event event,
-					void *user_data)
+static void known_networks_watch_cb(const char *filename,
+                                                enum l_dir_watch_event event,
+                                                void *user_data)
 {
 	const char *ssid;
 	L_AUTO_FREE_VAR(char *, full_path) = NULL;
@@ -318,11 +322,9 @@ static void known_networks_watch_cb(struct l_fswatch *watch,
 	full_path = storage_get_network_file_path(security, ssid);
 
 	switch (event) {
-	case L_FSWATCH_EVENT_DELETE:
-	case L_FSWATCH_EVENT_MOVE:
-	case L_FSWATCH_EVENT_MODIFY:
-	case L_FSWATCH_EVENT_ATTRIB:
-	case L_FSWATCH_EVENT_CREATE:
+	case L_DIR_WATCH_EVENT_CREATED:
+	case L_DIR_WATCH_EVENT_REMOVED:
+	case L_DIR_WATCH_EVENT_MODIFIED:
 		/*
 		 * For now treat all the operations the same.  E.g. they may
 		 * result in the removal of the network (file moved out, not
@@ -342,6 +344,9 @@ static void known_networks_watch_cb(struct l_fswatch *watch,
 		}
 
 		l_settings_free(settings);
+		break;
+	case L_DIR_WATCH_EVENT_ACCESSED:
+		break;
 	}
 }
 
@@ -364,9 +369,10 @@ bool known_networks_init(void)
 		return false;
 	}
 
-	dir = opendir(STORAGEDIR);
+	dir = opendir(DAEMON_STORAGEDIR);
 	if (!dir) {
-		l_info("Unable to open %s: %s", STORAGEDIR, strerror(errno));
+		l_info("Unable to open %s: %s", DAEMON_STORAGEDIR,
+							strerror(errno));
 		l_dbus_unregister_interface(dbus, IWD_KNOWN_NETWORK_INTERFACE);
 		return false;
 	}
@@ -399,8 +405,8 @@ bool known_networks_init(void)
 
 	closedir(dir);
 
-	storage_dir_watch = l_fswatch_new(STORAGEDIR, known_networks_watch_cb,
-						NULL,
+	storage_dir_watch = l_dir_watch_new(DAEMON_STORAGEDIR,
+						known_networks_watch_cb, NULL,
 						known_networks_watch_destroy);
 
 	return true;
@@ -410,6 +416,8 @@ void known_networks_exit(void)
 {
 	struct l_dbus *dbus = dbus_get_bus();
 
+	l_dir_watch_destroy(storage_dir_watch);
+
 	l_queue_destroy(known_networks, network_info_free);
 	known_networks = NULL;
 
