@@ -2,7 +2,7 @@
  *
  *  Wireless daemon for Linux
  *
- *  Copyright (C) 2013-2014  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2013-2018  Intel Corporation. All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -26,8 +26,20 @@
 struct wiphy;
 struct scan_bss;
 
+enum wiphy_state_watch_event {
+	WIPHY_STATE_WATCH_EVENT_POWERED,
+	WIPHY_STATE_WATCH_EVENT_RFKILLED,
+};
+
+typedef void (*wiphy_state_watch_func_t)(struct wiphy *wiphy,
+					enum wiphy_state_watch_event event,
+					void *user_data);
+typedef void (*wiphy_destroy_func_t)(void *user_data);
+
 enum ie_rsn_cipher_suite wiphy_select_cipher(struct wiphy *wiphy,
 							uint16_t mask);
+enum ie_rsn_akm_suite wiphy_select_akm(struct wiphy *wiphy,
+					struct scan_bss *bss);
 
 struct wiphy *wiphy_find(int wiphy_id);
 
@@ -40,6 +52,11 @@ uint8_t wiphy_get_max_num_ssids_per_scan(struct wiphy *wiphy);
 bool wiphy_supports_iftype(struct wiphy *wiphy, uint32_t iftype);
 bool wiphy_supports_adhoc_rsn(struct wiphy *wiphy);
 
+uint32_t wiphy_state_watch_add(struct wiphy *wiphy,
+				wiphy_state_watch_func_t func, void *user_data,
+				wiphy_destroy_func_t destroy);
+bool wiphy_state_watch_remove(struct wiphy *wiphy, uint32_t id);
+
 bool wiphy_init(struct l_genl_family *in, const char *whitelist,
 							const char *blacklist);
 bool wiphy_exit(void);
