@@ -30,7 +30,6 @@
 
 #include "src/dbus.h"
 #include "src/netdev.h"
-#include "src/watchlist.h"
 #include "src/wiphy.h"
 #include "src/station.h"
 #include "src/scan.h"
@@ -444,7 +443,7 @@ static void wsc_connect(struct wsc *wsc)
 		if (l_dbus_message_get_arguments(wsc->pending, "s", &pin)) {
 			enum wsc_device_password_id dpid;
 
-			if (wsc_pin_is_checksum_valid(pin))
+			if (strlen(pin) == 4 || wsc_pin_is_checksum_valid(pin))
 				dpid = WSC_DEVICE_PASSWORD_ID_DEFAULT;
 			else
 				dpid = WSC_DEVICE_PASSWORD_ID_USER_SPECIFIED;
@@ -561,8 +560,8 @@ static void pin_timeout(struct l_timeout *timeout, void *user_data)
 					wsc_error_time_expired(wsc->pending));
 }
 
-static bool push_button_scan_results(uint32_t wiphy_id, uint32_t ifindex,
-					int err, struct l_queue *bss_list,
+static bool push_button_scan_results(uint32_t ifindex, int err,
+					struct l_queue *bss_list,
 					void *userdata)
 {
 	struct wsc *wsc = userdata;
@@ -719,7 +718,7 @@ static bool authorized_macs_contains(const uint8_t *authorized_macs,
 	return false;
 }
 
-static bool pin_scan_results(uint32_t wiphy_id, uint32_t ifindex, int err,
+static bool pin_scan_results(uint32_t ifindex, int err,
 				struct l_queue *bss_list, void *userdata)
 {
 	static const uint8_t wildcard_address[] =
@@ -958,7 +957,7 @@ static struct l_dbus_message *wsc_start_pin(struct l_dbus *dbus,
 	if (!wsc_pin_is_valid(pin))
 		return dbus_error_invalid_format(message);
 
-	if (wsc_pin_is_checksum_valid(pin))
+	if (strlen(pin) == 4 || wsc_pin_is_checksum_valid(pin))
 		dpid = WSC_DEVICE_PASSWORD_ID_DEFAULT;
 	else
 		dpid = WSC_DEVICE_PASSWORD_ID_USER_SPECIFIED;
@@ -1080,15 +1079,11 @@ static void wsc_remove_interface(struct netdev *netdev)
 static void wsc_netdev_watch(struct netdev *netdev,
 				enum netdev_watch_event event, void *userdata)
 {
-	struct device *device = netdev_get_device(netdev);
-
-	if (!device)
-		return;
-
 	switch (event) {
 	case NETDEV_WATCH_EVENT_UP:
 	case NETDEV_WATCH_EVENT_NEW:
-		if (netdev_get_iftype(netdev) == NETDEV_IFTYPE_STATION)
+		if (netdev_get_iftype(netdev) == NETDEV_IFTYPE_STATION &&
+				netdev_get_is_up(netdev))
 			wsc_add_interface(netdev);
 		break;
 	case NETDEV_WATCH_EVENT_DOWN:
