@@ -32,7 +32,6 @@
 #include <stdlib.h>
 #include <stdbool.h>
 #include <stdint.h>
-#include <signal.h>
 #include <string.h>
 #include <getopt.h>
 #include <poll.h>
@@ -530,20 +529,31 @@ static void create_dbus_system_conf(void)
 
 static bool start_dbus_daemon(void)
 {
-	char *argv[3];
+	char *argv[4];
 	pid_t pid;
 
 	argv[0] = "dbus-daemon";
 	argv[1] = "--system";
-	argv[2] = NULL;
+	argv[2] = "--nosyslog";
+	argv[3] = NULL;
 
-	pid = execute_program(argv, false, false);
+	if (check_verbosity("dbus"))
+		setenv("DBUS_VERBOSE", "1", true);
+
+	pid = execute_program(argv, false, check_verbosity("dbus"));
 	if (pid < 0)
 		return false;
 
 	if (!wait_for_socket("/run/dbus/system_bus_socket", 25 * 10000))
 		return false;
 
+	if (check_verbosity("dbus-monitor")) {
+		argv[0] = "dbus-monitor";
+		argv[1] = "--system";
+		argv[2] = NULL;
+		execute_program(argv, false, true);
+	}
+
 	l_debug("D-Bus is running");
 
 	return true;
@@ -1470,8 +1480,7 @@ static void test_timeout_timer_tick(struct l_timeout *timeout, void *user_data)
 	l_main_quit();
 }
 
-static void test_timeout_signal_handler(struct l_signal *signal, uint32_t signo,
-								void *user_data)
+static void test_timeout_signal_handler(uint32_t signo, void *user_data)
 {
 	switch (signo) {
 	case SIGINT:
@@ -1484,8 +1493,6 @@ static void test_timeout_signal_handler(struct l_signal *signal, uint32_t signo,
 static pid_t start_execution_timeout_timer(unsigned int max_exec_interval_sec,
 							pid_t *test_exec_pid)
 {
-	sigset_t mask;
-	struct l_signal *signal;
 	struct l_timeout *test_exec_timeout;
 	pid_t test_timer_pid;
 
@@ -1499,22 +1506,15 @@ static pid_t start_execution_timeout_timer(unsigned int max_exec_interval_sec,
 		if (!l_main_init())
 			exit(EXIT_FAILURE);
 
-		sigemptyset(&mask);
-		sigaddset(&mask, SIGINT);
-		sigaddset(&mask, SIGTERM);
-
-		signal = l_signal_create(&mask, test_timeout_signal_handler,
-							test_exec_pid, NULL);
 		test_exec_timeout =
 			l_timeout_create(max_exec_interval_sec,
 						test_timeout_timer_tick,
 						test_exec_pid,
 						NULL);
 
-		l_main_run();
+		l_main_run_with_signal(test_timeout_signal_handler, NULL);
 
 		l_timeout_remove(test_exec_timeout);
-		l_signal_remove(signal);
 
 		l_main_exit();
 
@@ -1782,20 +1782,23 @@ static void create_network_and_run_tests(const void *key, void *value,
 		setenv("IWD_TEST_TIMEOUTS", "off", true);
 
 	if (!create_tmpfs_extra_stuff(tmpfs_extra_stuff))
-		goto exit_hwsim;
+		goto remove_abs_paths;
 
 	if (!configure_hw_radios(hw_settings, wiphy_list))
-		goto exit_hwsim;
+		goto remove_abs_paths;
 
 	medium_pid = register_hwsim_as_trans_medium();
 	if (medium_pid < 0)
-		goto exit_hwsim;
+		goto remove_abs_paths;
 
 	if (check_verbosity("hwsim")) {
 		list_hwsim_radios();
 		list_interfaces();
 	}
 
+	if (check_verbosity("tls"))
+		setenv("IWD_TLS_DEBUG", "on", true);
+
 	if (!configure_hostapd_instances(hw_settings, config_dir_path,
 						wiphy_list, hostapd_pids))
 		goto exit_hostapd;
@@ -1848,11 +1851,13 @@ exit_hostapd:
 
 	terminate_medium(medium_pid);
 
+remove_abs_paths:
+	remove_absolute_path_dirs(tmpfs_extra_stuff);
+
 exit_hwsim:
 	l_queue_destroy(wiphy_list, wiphy_free);
 
 	l_settings_free(hw_settings);
-	remove_absolute_path_dirs(tmpfs_extra_stuff);
 	l_strfreev(tmpfs_extra_stuff);
 }
 
@@ -1918,6 +1923,7 @@ static void print_results(struct l_queue *test_stat_queue)
 	char sep_line[80];
 
 	memset(sep_line, '_', sizeof(sep_line) - 1);
+	sep_line[sizeof(sep_line) - 1] = '\0';
 
 	l_info("%s\n" CONSOLE_LN_RESET, sep_line);
 	l_info("%27s " CONSOLE_LN_DEFAULT "|" CONSOLE_LN_GREEN " %s "
@@ -1927,6 +1933,7 @@ static void print_results(struct l_queue *test_stat_queue)
 		"Configuration cycle", "PASSED", "FAILED", "TIMED OUT");
 
 	memset(sep_line, '-', sizeof(sep_line) - 1);
+	sep_line[sizeof(sep_line) - 1] = '\0';
 	l_info("%s" CONSOLE_LN_RESET, sep_line);
 
 	l_queue_foreach(test_stat_queue, print_test_stat, &stat_totals);
@@ -1941,6 +1948,7 @@ static void print_results(struct l_queue *test_stat_queue)
 			stat_totals.total_timedout, stat_totals.total_duration);
 
 	memset(sep_line, '_', sizeof(sep_line) - 1);
+	sep_line[sizeof(sep_line) - 1] = '\0';
 	l_info("%s" CONSOLE_LN_RESET, sep_line);
 }
 
